var tape       = require('tape'),
    R          = require('ramda'),
    mapper     = require('../index'),
    sampleData = require('./sample-data.json');


tape.test('key mapper should return the correct key name', assert => {
    const table = {
        /* 'common' webapp data */
        "name":        {
            "id":     "ItemName",
            "bcType": "itemName"
        },
        "description": {
            "id":     "ItemDescription",
            "bcType": "itemName"
        },
        "keywords":    {
            "id":     "CAT_txtKeywords",
            "bcType": null
        },
        "category":    {
            "id":     "CAT_Category",
            "bcType": null
        },

        /* 'local' webapp data (varies by webapp) */
        firstCustomField:      {
            "id":     "CAT_Custom_1",
            "bcType": "textString"
        },
        'Second Custom Field': {
            "id":     "CAT_Custom_2",
            "bcType": "textString"
        }
    };

    assert.deepEqual(
        mapper.getNewKey(table, 'This Field Is Not Mapped'),
        'This Field Is Not Mapped',
        'keys not in table should be preserved'
    );

    assert.deepEqual(
        mapper.getNewKey(table, 'firstCustomField'),
        'CAT_Custom_1',
        'keys in table should be replaced'
    );

    assert.end();
});


tape.test('key replacer should replace keys when possible', assert => {
    assert.deepEqual(
        mapper.mapKeys(R.toUpper, {a: 1, b: 2, c: 3}),
        {A: 1, B: 2, C: 3},
        'cookbook example'
    );

    assert.end();
});


tape.test('mapping should work as expected', assert => {
    /*
     note this is 'local' webapp data merged with the 'common' webapp data

     tip: use `R.merge(commonMap, localMap)`.
     */
    const targetWebapp = {
        /* 'common' webapp data */
        "name":        {
            "id":     "ItemName",
            "bcType": "itemName"
        },
        "description": {
            "id":     "ItemDescription",
            "bcType": "itemName"
        },
        "keywords":    {
            "id":     "CAT_txtKeywords",
            "bcType": null
        },
        "category":    {
            "id":     "CAT_Category",
            "bcType": null
        },

        /* 'local' webapp data (varies by webapp) */
        firstCustomField:      {
            "id":     "CAT_Custom_1",
            "bcType": "textString"
        },
        'Second Custom Field': {
            "id":     "CAT_Custom_2",
            "bcType": "textString"
        }
    };

    assert.deepEqual(
        mapper.mapFields(targetWebapp, {
            firstCustomField:      'theFirstCustomField',
            'Second Custom Field': 'theSecondCustomField'
        }),
        {
            'CAT_Custom_1': 'theFirstCustomField',
            'CAT_Custom_2': 'theSecondCustomField'
        },
        'basic 1-to-1 mapping'
    );

    assert.deepEqual(
        mapper.mapFields(targetWebapp)({
            firstCustomField:      'theFirstCustomField',
            'Second Custom Field': 'theSecondCustomField'
        }),
        {
            'CAT_Custom_1': 'theFirstCustomField',
            'CAT_Custom_2': 'theSecondCustomField'
        },
        'basic 1-to-1 mapping, curried'
    );

    assert.deepEqual(
        mapper.mapFields(targetWebapp, {
            name:                  'theName',
            description:           'theDescription',
            firstCustomField:      'theFirstCustomField',
            'Second Custom Field': 'theSecondCustomField'
        }),
        {
            ItemName:        'theName',
            ItemDescription: 'theDescription',
            'CAT_Custom_1':  'theFirstCustomField',
            'CAT_Custom_2':  'theSecondCustomField'
        },
        'basic 1-to-1 mapping, including "common" fields'
    );

    assert.deepEqual(
        mapper.mapFields(targetWebapp, {
            name:                       'theName',
            description:                'theDescription',
            firstCustomField:           'theFirstCustomField',
            'Second Custom Field':      'theSecondCustomField',
            'This Field Is Not Mapped': 'This Field Is Not Mapped'
        }),
        {
            ItemName:                   'theName',
            ItemDescription:            'theDescription',
            'CAT_Custom_1':             'theFirstCustomField',
            'CAT_Custom_2':             'theSecondCustomField',
            'This Field Is Not Mapped': 'This Field Is Not Mapped'
        },
        'fields with no mapping should be preserved'
    );

    assert.end();
});


tape.test('convenience function should return combined map', assert => {

    const commonConfig = sampleData.commonWebappConfigFile,
          appConfig  = sampleData.localWebappConfig,
          mergedMap = sampleData.mergedWebappFieldMap;

    assert.deepEqual(
        mapper.extractCombinedFieldMap(commonConfig, appConfig),
        mergedMap,
        'basic map merging'
    );

    assert.deepEqual(
        mapper.extractCombinedFieldMap(commonConfig)(appConfig),
        mergedMap,
        'basic map merging, curried'
    );

    assert.end();
});

// TODO: mapper should reject invalid map json

// TODO: convenience merger should reject invalid webapp-config json